/*
  Java gfx-test framework

   Copyright (C) 2010, 2011, 2012  Red Hat

This file is part of IcedTea.

IcedTea is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2, or (at your option)
any later version.

IcedTea is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with IcedTea; see the file COPYING.  If not, write to the
Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
02110-1301 USA.

Linking this library statically or dynamically with other modules is
making a combined work based on this library.  Thus, the terms and
conditions of the GNU General Public License cover the whole
combination.

As a special exception, the copyright holders of this library give you
permission to link this library with independent modules to produce an
executable, regardless of the license terms of these independent
modules, and to copy and distribute the resulting executable under
terms of your choice, provided that you also meet, for each linked
independent module, the terms and conditions of the license of that
module.  An independent module is a module which is not derived from
or based on this library.  If you modify this library, you may extend
this exception to your version of the library, but you are not
obligated to do so.  If you do not wish to do so, delete this
exception statement from your version.
*/

package org.gfxtest.harness;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;



import org.gfxtest.framework.GfxTestConfiguration;

/**
 * This class contains methods used for run external commands: compile selected
 * test suite and run selected test.
 * 
 * @author Pavel Tisnovsky
 */
public class ExternalCommands
{
    /**
     * Default encoding used for all input/output operations.
     */
    private static final String DEFAULT_ENCODING = "UTF-8";

    /**
     * Run options used to start each test suite.
     */
    private static final String RUN_OPTIONS =
        "-w=" + GfxTestConfiguration.DEFAULT_IMAGE_WIDTH + " " +
        "-h=" + GfxTestConfiguration.DEFAULT_IMAGE_HEIGHT + " " +
        "-t=" + GfxTestConfiguration.DEFAULT_IMAGE_TYPE;
    
    /**
     * Read all lines from buffered reader and converts all lines to one string.
     * 
     * @param bufferedReader
     *            reader used to read text lines
     * @return all lines concatenated into one string
     * @throws IOException
     */
    private static String copyFromBufferedReaderToString(BufferedReader bufferedReader) throws IOException
    {
        String line;
        StringBuffer output = new StringBuffer();
        while ((line = bufferedReader.readLine()) != null)
        {
            output.append(line + "\n");
        }
        return output.toString();
    }

    /**
     * Read standard output from the command and put it to String
     * 
     * @param process
     *            compile or run process
     * @return standard output from given process
     * @throws IOException
     */
    private static String fetchStandardOutput(Process process) throws IOException
    {
        BufferedReader stdInput = new BufferedReader(new InputStreamReader(process.getInputStream(), DEFAULT_ENCODING));
        return copyFromBufferedReaderToString(stdInput);
    }

    /**
     * Read error output from the command and put it to String
     * 
     * @param process
     *            compile or run process
     * @return error output from given process
     * @throws IOException
     */
    private static String fetchErrorOutput(Process process) throws IOException
    {
        BufferedReader stdError = new BufferedReader(new InputStreamReader(process.getErrorStream(), DEFAULT_ENCODING));
        return copyFromBufferedReaderToString(stdError);
    }

    /**
     * This method constructs string containing external command to build
     * (compile) selected test suite.
     * 
     * @param selectedTestSuite
     *            name of selected test suite
     * @param verboseOperation
     *            whether to use verbose operation during compile
     * @return string containing external command
     */
    private static String constructBuildCommand(String selectedTestSuite, boolean verboseOperation)
    {
        return "javac -d " + Paths.BUILD_DIRECTORY + (verboseOperation ? " -verbose " : "") + " -sourcepath "
                + Paths.SOURCE_DIRECTORY + "/ " + Paths.PATH_TO_TESTSUITES + "/" + selectedTestSuite + ".java";
    }

    /**
     * This method constructs string containing external command to run selected
     * test suite.
     * 
     * @param pathToJava
     *            path to selected JVM (may be tested JVM or reference one)
     * @param selectedTestName
     *            name of selected test suite
     * @param outputDirectoryForSelectedTest
     *            name of directory, where the output files might be created
     * @param verboseOperation
     *            whether to use verbose operation during test run
     * @return string containing external command
     */
    private static String constructRunCommand(String pathToJava, String selectedTestName, String outputDirectoryForSelectedTest, boolean verboseOperation)
    {
        return pathToJava + (verboseOperation ? " -verbose " : "") + " -cp " + Paths.BUILD_DIRECTORY
                + " org.gfxtest.testsuites." + selectedTestName + " " + RUN_OPTIONS +
                " -o=" + outputDirectoryForSelectedTest;
    }

    /**
     * This method tries to compile selected test suite.
     * 
     * @param selectedTestSuite
     *            name of selected test suite
     * @param verboseOperation
     *            whether to use verbose operation during test run
     * @return standard and error output of external process
     * @throws IOException
     * @throws InterruptedException
     * @throws RuntimeException
     */
    public static String compileSelectedTestSuite(String selectedTestSuite, boolean verboseOperation) throws IOException, InterruptedException, RuntimeException
    {
        FileSystemTools.createDirectory(Paths.BUILD_DIRECTORY);
        String compilationCommand = constructBuildCommand(selectedTestSuite, verboseOperation);
        Process process = Runtime.getRuntime().exec(compilationCommand);
        StringBuffer output = new StringBuffer();
        appendCompilationMessageHeader(compilationCommand, output);
        output.append(fetchStandardOutput(process));
        appendErrorMessageHeader(output);
        output.append(fetchErrorOutput(process));
        // wait for the external process to finish
        process.waitFor();
        // check external process's exit value
        if (process.exitValue() != 0)
        {
            throw new RuntimeException("Exit code = " + process.exitValue());
        }
        return output.toString();
    }

    /**
     * This method tries to run selected test suite on selected JDK (or to be
     * more precise - JVM)
     * 
     * @param pathToJava
     *            path to selected JVM (may be tested JVM or reference one)
     * @param selectedTestName
     *            name of selected test suite
     * @param outputDirectory
     *            name of directory, where the output files might be created
     * @param verboseOperation
     *            whether to use verbose operation during test run
     * @return standard and error output of external process
     * @throws IOException
     * @throws InterruptedException
     * @throws RuntimeException
     */
    public static String runTestOnGivenJDK(String pathToJava, String selectedTestName, String outputDirectory, boolean verboseOperation) throws IOException, InterruptedException, RuntimeException
    {
        String outputDirectoryForSelectedTest = FileSystemTools.constructTestDirectoryName(outputDirectory, selectedTestName);
        FileSystemTools.createDirectory(outputDirectory);
        FileSystemTools.createDirectory(outputDirectoryForSelectedTest);
        FileSystemTools.eraseDirectory(outputDirectoryForSelectedTest);
        String runCommand = constructRunCommand(pathToJava, selectedTestName, outputDirectoryForSelectedTest, verboseOperation);
        Process process = Runtime.getRuntime().exec(runCommand);
        StringBuffer output = new StringBuffer();
        appendRunMessageHeader(runCommand, output);
        output.append(fetchStandardOutput(process));
        appendErrorMessageHeader(output);
        output.append(fetchErrorOutput(process));
        // wait for the external process to finish
        process.waitFor();
        // check external process's exit value
        if (process.exitValue() != 0)
        {
            throw new RuntimeException("Exit code = " + process.exitValue());
        }
        return output.toString();
    }

    /**
     * Append basic information about compile process to a given output stored
     * in a StringBuffer.
     * 
     * @param compilationCommand
     *            compilation command
     * @param output
     *            output stored in StringBuffer.
     */
    private static void appendCompilationMessageHeader(String compilationCommand, StringBuffer output)
    {
        output.append("Compilation command:\n");
        output.append(compilationCommand + "\n");
        output.append("\n");
        output.append("Standard output for compilation: \n");
        output.append("---------------------------------\n");
    }

    /**
     * Append basic information about compile and/or run failures to a given
     * output stored in a StringBuffer.
     * 
     * @param output
     *            output stored in StringBuffer.
     */
    private static void appendErrorMessageHeader(StringBuffer output)
    {
        output.append("\n");
        output.append("Error output: \n");
        output.append("---------------------------------\n");
    }

    /**
     * Append basic information test run to a given output stored in a
     * StringBuffer.
     * 
     * @param runCommand
     *            run command
     * @param output
     *            output stored in StringBuffer.
     */
    private static void appendRunMessageHeader(String runCommand, StringBuffer output)
    {
        output.append("Run command:\n");
        output.append(runCommand + "\n");
        output.append("\n");
        output.append("Standard output for test run: \n");
        output.append("---------------------------------\n");
    }

}
